#include "stdafx.h"
#include "RN.h"

namespace MyLib {
	using namespace System;

	double sqr( double x ){
					return x * x;
			}

	//
	//    Wichmann/Hill generator... = 1.0e12
	//
	WHrn::WHrn( int vc, int vs ): x(11111), y(22222), z(33333), c(vc), s(vs) {
		if (s > 0)
			for (int i = 0; i < s; i++) 
				uni0();
	}
	WHrn::WHrn( int vc, int vs, unsigned __int32 ix, unsigned __int32 iy, unsigned __int32 iz):
					x(ix), y(iy), z(iz), c(vc), s(vs) {
		if (s > 0)
			for (int i = 0; i < s; i++)
				uni0();
	}

	void WHrn::uni0(){
		do {
			x = (171 * x) % 30269;
			y = (172 * y) % 30307;
			z = (170 * z) % 30323;
			u = (x/30269.0) + (y/30307.0) + (z/30323.0);
			u = u - Math::Truncate(u);
		} while (u <= 0.0);
		return;
	}
	double WHrn::uni(){
		uni0();
		if (c > 1)
			for (int i = 1; i < c; i++)
				uni0();
		return u;
	}

	//
	//		A Rejection Polar Method for Normal Variates
	//
	double NormalRN::normal(){
		double v1, v2, w;
		do {
			v1 = 2.0 * uni() - 1.0;
			v2 = 2.0 * uni() - 1.0;
			w  = sqr(v1) + sqr(v2);
		} while (((w >= 1.0) || (w <= 0.0)));

		double c = Math::Sqrt(-2.0 * Math::Log(w)/w);

		return c * v1;
	}

	double NormalRN::normalMS(double m, double s){
			double v = s * normal() + m;
			return  v;
	}
	//
	//		A Rejection Polar Method for Normal Variates
	//
	void  NormalRN::normalPair( double & n1, double & n2 ){
			double v1, v2, w, c;
			do {
				  v1 = 2.0 * uni() - 1.0;
				  v2 = 2.0 * uni() - 1.0;
				  w  = sqr(v1) + sqr(v2);
			} while (((w >= 1.0) || (w <= 0.0)));

			c = Math::Sqrt(-2.0 * Math::Log(w) / w);

			n1 = c * v1;
			n2 = c * v2;
	}

}
